/* Copyright 2008 Gentoo Foundation
 * Distributed under the terms of the GNU General Public License v2
 * Author: Michael Haubenwallner <haubi@gentoo.org>
 */

#ifndef _aixdll_Object_h
#define _aixdll_Object_h

#include <string>
#include <vector>

#include "smartptr.h"

#include "ObjectKey.h"

namespace aixdll {

class Object : virtual public RefCounted
{
public:
	class Flags {
#undef FLAGS
#define FLAGS \
	FLAG(SHROBJ) \
	FLAG(EXEC) \
	FLAG(DYNLOAD) \
	FLAG(DEP_SYSTEM) \
	FLAG(DEP__SYSTEM) \
	FLAG(RELFLG) \
	FLAG(LNNO) \
/* EOF */

	public:

		enum type {
#undef FLAG
#define FLAG(flag) flag,
			FLAGS
			Unknown
		};

		static type getFlag(std::string const& name)
		{
#undef FLAG
#define FLAG(flag) if (name == #flag) return flag;
			FLAGS
			return Unknown;
		}

		static std::string getName(type that)
		{
#undef FLAG
#define FLAG(flag) if (flag == that) return std::string(#flag);
			FLAGS
			return std::string("Unknown");
		}
	};

	virtual ~Object() {}

	Object(ObjectKeyPtr const& key
		, std::string const& flags
		, std::string const& format
	) : key_(key)
	  , flags_()
	  , format_(format)
	  , resolved_()
	{
		std::string::const_iterator start = flags.begin();
		std::string::const_iterator end = start;
		std::string::const_iterator current = end;
		while(end != flags.end()) {
			current = end++;
			if (*current == ' ') {
				if (current > start) {
					flags_.push_back(Flags::getFlag(std::string(start, current)));
				}
				start = end;
			}
		}
	}

	ObjectKeyPtr const& key() const
	{ return key_; }

	void setResolved(std::string const& resolved)
	{ resolved_ = resolved; }

	std::string const& getResolved() const
	{ return resolved_; }

	std::string flags() const
	{
		std::string out;
		for(std::vector<Flags::type>::const_iterator f = flags_.begin()
		  ; f != flags_.end()
		  ; f++
		) {
			out += " ";
			out += Flags::getName(*f);
		}
		return out;
	}

	std::string const& format() const
	{ return format_; }

protected:
	ObjectKeyPtr        key_;

private:
	std::vector<Flags::type> flags_;
	std::string         format_;
	std::string			resolved_;
};

typedef SmartPtr<Object> ObjectPtr;

}

#endif /* _aixdll_Object_h */
