/* Copyright 2008 Gentoo Foundation
 * Distributed under the terms of the GNU General Public License v2
 * Author: Michael Haubenwallner <haubi@gentoo.org>
 */

#ifndef _aixdll_SearchPath_h
#define _aixdll_SearchPath_h

#include <string>
#include <vector>

#include "smartptr.h"

namespace aixdll {

class SearchPath : virtual public RefCounted
{
public:
	typedef std::vector<std::string> PathContainer;

	~SearchPath() {}

	SearchPath()
		: components_()
	{}

	SearchPath(std::string const& separated)
		: components_()
	{
		appendSeparated(separated);
	}

	void appendSeparated(std::string const& separated)
	{
		std::string::const_iterator start = separated.begin();
		std::string::const_iterator end = start;
		std::string::const_iterator current = end;
		while(end != separated.end()) {
			current = end++;
			if (*current == ':') {
				if (*(current - 1) == '/') {
					components_.push_back(std::string(start, current));
				} else {
					components_.push_back(std::string(start, current) + "/");
				}
				start = end;
			}
		}
		if (current > start) {
			if (*(end-1) == '/') {
				components_.push_back(std::string(start, end));
			} else {
				components_.push_back(std::string(start, end) + "/");
			}
		}
	}

	void append(SearchPath const& that)
	{
		components_.insert(components_.end(), that.components_.begin(), that.components_.end());
	}

	PathContainer const& components() const
	{ return components_; }

	std::string asSeparated() const
	{
		std::string separated;
		char const *sep = "";
		for(PathContainer::const_iterator p = components_.begin()
		  ; p != components_.end()
		  ; p++
		) {
			separated += sep;
			separated += *p;
			if (p->length() > 1 && (*p)[p->length()-1] == '/') {
				separated.erase(separated.length()-1);
			}
			sep = ":";
		}
		return separated;
	}
private:
	PathContainer components_;
};

typedef SmartPtr<SearchPath> SearchPathPtr;

}

#endif /* _aixdll_SearchPath_h */
