/* Copyright 2008 Gentoo Foundation
 * Distributed under the terms of the GNU General Public License v2
 * Author: Michael Haubenwallner <haubi@gentoo.org>
 */

#ifndef _aixdll_SharedObject_h
#define _aixdll_SharedObject_h

#include "Object.h"
#include "Dependency.h"

namespace aixdll {

class ObjectReader;

class SharedObject : public Object
{
public:
	typedef std::vector<DependencyPtr> DependencyContainer;

	~SharedObject() {}

	SharedObject(ObjectKeyPtr const& key
		, std::string const& flags
		, std::string const& format
	) : Object(key, flags, format)
	  , runpath_()
	  , dependencies_()
	{}

	DependencyContainer const & getDependencies() const
	{ return dependencies_; }

	SearchPathPtr const& runpath() const
	{ return runpath_; }
protected:
	friend class ObjectReader;
	void setRunpath(SearchPathPtr const& runpath)
	{
		runpath_ = runpath;
	}

	void addDependency(DependencyPtr const& dependency)
	{
		dependencies_.push_back(dependency);
	}
private:
	SearchPathPtr  runpath_;
	DependencyContainer dependencies_;
};

typedef SmartPtr<SharedObject> SharedObjectPtr;

}

#endif /* _aixdll_SharedObject_h */
