/* Copyright 2008 Gentoo Foundation
 * Distributed under the terms of the GNU General Public License v2
 * Author: Michael Haubenwallner <haubi@gentoo.org>
 */

#include <string>
#include <map>
#include <iostream>
#include <errno.h>
#include <string.h>

#include "ObjectReader.h"

using namespace aixdll;

typedef ObjectReader::ObjectContainer ObjectContainer;
typedef std::map<std::string, bool> QuestionContainer;

bool query(std::string const& filename, QuestionContainer const& questions)
{
	ObjectReader reader;
	ObjectContainer objects;

	if (reader.readObjects(filename, filename, objects) == 0) {
		std::cerr << "ldd: " << filename << ": " << strerror(ENOENT) << std::endl;
		return true;
	}

	for(ObjectContainer::iterator o = objects.begin()
	  ; o != objects.end()
	  ; o++
	) {
		if (questions.find("FILE") != questions.end()) {
				std::cout << " FILE='";
				if ((*o)->key()->path()) {
					std::cout << (*o)->key()->path()->asIdentifier();
				}
				std::cout << (*o)->key()->name()->base() << "';";
		}
		if (questions.find("MEMBER") != questions.end()) std::cout
			<< " MEMBER='"
				<< (*o)->key()->name()->member()
			<< "';"
		;
		if (questions.find("FLAGS") != questions.end()) std::cout
			<< " FLAGS='"
				<< (*o)->flags()
			<< "';"
		;
		if (questions.find("FORMAT") != questions.end()) std::cout
			<< " FORMAT='"
				<< (*o)->format()
			<< "';"
		;
		SharedObject *sharedObject = dynamic_cast<SharedObject*>(o->ptr());
		StaticObject *staticObject = dynamic_cast<StaticObject*>(o->ptr());
		if (sharedObject) {
			if (questions.find("SHROBJ") != questions.end()) std::cout
				<< " SHROBJ=true;"
			;
			if (questions.find("RUNPATH") != questions.end()) std::cout
				<< " RUNPATH='"
					<< sharedObject->runpath()->asSeparated()
				<< "';"
			;
			if (questions.find("DEPLIBS") != questions.end()) {
				std::cout
					<< " DEPLIBS=\""
				;
				SharedObject::DependencyContainer const& deps = sharedObject->getDependencies();
				for(SharedObject::DependencyContainer::const_iterator dep = deps.begin()
				  ; dep != deps.end()
				  ; dep++
				) {
					std::cout << " '";
					if ((*dep)->key()) {
						std::cout << (*dep)->key()->asIdentifier();
					} else {
						std::cout << (*dep)->name()->asIdentifier();
					}
					std::cout << "'";
				}
				std::cout << "\";";
			}
		} else {
			if (questions.find("SHROBJ") != questions.end()) std::cout
				<< " SHROBJ=false;"
			;
			if (questions.find("RUNPATH") != questions.end()) std::cout
				<< " RUNPATH='';"
			;
			if (questions.find("DEPLIBS") != questions.end()) std::cout
				<< " DEPLIBS='';"
			;
		}

		std::cout << std::endl;
	}

	return false;
}

int main(int argc, char **argv)
{
	int i;
	QuestionContainer questions;

	if (argc < 2) {
		std::cout << argv[0] << " file [what1 [what2] [whatn]]\n"
			"available for [what]:\n"
			"\tFILE MEMBER FLAGS FORMAT SHROBJ RUNPATH DEPLIBS"
		<< std::endl;
		return 0;
	}

	for(i = 2; i < argc; i++) {
		questions[argv[i]] = true;
	}

	if (questions.size() == 0) {
		questions["FILE"] = true;
		questions["MEMBER"] = true;
		questions["FLAGS"] = true;
		questions["FORMAT"] = true;
		questions["SHROBJ"] = true;
		questions["RUNPATH"] = true;
		questions["DEPLIBS"] = true;
	}

	return query(argv[1], questions);
}
